package madqba.awt;

import madqba.awt.chat.Mediator;

import java.awt.Color;
import java.awt.TextField;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;


/**
 * This class is an extended TextFiled. Every time user types in a character,
 * it's checked if it makes a valid IRC nickname. If not, the character is
 * removed.
 *
 * @author Jakub Madej
 */
public class Nicktf extends TextField implements TextListener, KeyListener {
    private Mediator med;

    /**
     * Creates a new Nicktf object.
     *
     * @param med Mediator object
     */
    public Nicktf(Mediator med) {
        this.med = med;
        setBackground(new Color(255, 255, 255));
        addTextListener(this);
        addKeyListener(this);
    }

    /**
     * Invoked when the value of the text has changed.
     * This method validates the contents of TextFiled, so it contains only
     * valid characters.
     *
     * @param e TextEvent object
     */
    public void textValueChanged(TextEvent e) {
        String text = getText();
        int carpos = getCaretPosition();

        if (text.length() == 0)
            med.polaczSetEnabled(false);
        else
            med.polaczSetEnabled(true);

        if (text.length() > 9) {
            text = text.substring(0, carpos - 1) + text.substring(carpos);
            setText(text);
            setCaretPosition(--carpos);
        } else if ((carpos == 1) && !Character.isLetter(text.charAt(0)) &&
                !text.startsWith("[") && !text.startsWith("]") &&
                !text.startsWith("\\") && !text.startsWith("`") &&
                !text.startsWith("_") && !text.startsWith("^") &&
                !text.startsWith("{") && !text.startsWith("|") &&
                !text.startsWith("}")) {
            text = text.substring(1);
            setText(text);
            setCaretPosition(--carpos);
        } else if ((carpos > 1) &&
                !Character.isLetterOrDigit(text.charAt(carpos - 1)) &&
                !text.substring(carpos - 1, carpos).equals("[") &&
                !text.substring(carpos - 1, carpos).equals("]") &&
                !text.substring(carpos - 1, carpos).equals("\\") &&
                !text.substring(carpos - 1, carpos).equals("`") &&
                !text.substring(carpos - 1, carpos).equals("_") &&
                !text.substring(carpos - 1, carpos).equals("^") &&
                !text.substring(carpos - 1, carpos).equals("]") &&
                !text.substring(carpos - 1, carpos).equals("{") &&
                !text.substring(carpos - 1, carpos).equals("|") &&
                !text.substring(carpos - 1, carpos).equals("}") &&
                !text.substring(carpos - 1, carpos).equals("-")) {
            text = text.substring(0, carpos - 1) + text.substring(carpos);
            setText(text);
            setCaretPosition(--carpos);
        }
    }

    /**
     * This method isn't used, but it has to be implemented.
     *
     * @param e KeyEvent object
     */
    public void keyTyped(KeyEvent e) {
    }

    /**
     * This method isn't used, but it has to be implemented.
     *
     * @param e KeyEvent object
     */
    public void keyReleased(KeyEvent e) {
    }

    /**
     * Invoked when a key has been pressed. This method is used to detect
     * pressing of enter key.
     *
     * @param e KeyEvent object
     */
    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == e.VK_ENTER)
            med.polacz();
    }
}
