package madqba.awt;

import madqba.awt.chat.Mediator;

import java.awt.*;
import java.awt.event.*;


/**
 * This class represents a menu containing 12 SmileButtons objects.
 *
 * @author Jakub Madej
 */
public class SMenu extends Container implements ActionListener {
    private static final int SIZE = 20;
    private SmileButton[] imgtab;
    private TextField tf;
    private Mediator med;

    /**
     * Creates a new SMenu object.
     *
     * @param med Mediator object
     * @param tf TextField to display selected smiles in
     */
    public SMenu(Mediator med, TextField tf) {
        imgtab = new SmileButton[] {
                new SmileButton(med.getSmiles().getSmile1()),
                new SmileButton(med.getSmiles().getSmile2()),
                new SmileButton(med.getSmiles().getSmile3()),
                new SmileButton(med.getSmiles().getSmile4()),
                new SmileButton(med.getSmiles().getSmile5()),
                new SmileButton(med.getSmiles().getSmile6()),
                new SmileButton(med.getSmiles().getSmile7()),
                new SmileButton(med.getSmiles().getSmile8()),
                new SmileButton(med.getSmiles().getSmile9()),
                new SmileButton(med.getSmiles().getSmile10()),
                new SmileButton(med.getSmiles().getSmile11()),
                new SmileButton(med.getSmiles().getSmile12())
            };
        this.med = med;
        this.tf = tf;
        setLayout(null);

        int j = 0;

        for (int i = 0; i < imgtab.length; i++) {
            imgtab[i].setBounds(SIZE * (i % 4), (i / 4) * SIZE, SIZE, SIZE);
            add(imgtab[i]);
            imgtab[i].addActionListener(this);
            j = i;
        }

        setSize(SIZE * 4, SIZE * (1 + (j / 4)));
    }

    /**
     * Shows or hides this component depending on the value of parameter b.
     *
     * @param b if true, shows this component; otherwise, hides this component
     */
    public void setVisible(boolean b) {
        super.setVisible(b);

        for (int i = 0; i < imgtab.length; i++)
            imgtab[i].setVisible(b);
    }

    /**
     * Invoked when an action occurs.
     *
     * @param e ActionEvent object
     */
    public void actionPerformed(ActionEvent e) {
        int i = 0;
        int pos = tf.getCaretPosition();

        while (imgtab[i] != e.getSource())
            i++;

        switch (i) {
        case 0:
            tf.setText(tf.getText().substring(0, pos) + " :-) " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 1:
            tf.setText(tf.getText().substring(0, pos) + " ;-( " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 2:
            tf.setText(tf.getText().substring(0, pos) + " :-| " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 3:
            tf.setText(tf.getText().substring(0, pos) + " :-D " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 4:
            tf.setText(tf.getText().substring(0, pos) + " :-( " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 5:
            tf.setText(tf.getText().substring(0, pos) + " 8^) " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 6:
            tf.setText(tf.getText().substring(0, pos) + " :O  " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 7:
            tf.setText(tf.getText().substring(0, pos) + " :-P " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 8:
            tf.setText(tf.getText().substring(0, pos) + " :-] " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 9:
            tf.setText(tf.getText().substring(0, pos) + " =)  " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 10:
            tf.setText(tf.getText().substring(0, pos) + " ??  " +
                tf.getText().substring(pos, tf.getText().length()));

            break;

        case 11:
            tf.setText(tf.getText().substring(0, pos) + " !!  " +
                tf.getText().substring(pos, tf.getText().length()));

            break;
        }

        tf.setCaretPosition(pos + 5);
    }
}
