package madqba.awt.chat;

import madqba.awt.ChatPanel;
import madqba.awt.SMenu;
import madqba.awt.SmileButton;
import madqba.awt.TextCommand;
import madqba.awt.TextEntry;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


/**
 * This class represent a Frame in which a private conversation takes place.
 *
 * @author Jakub Madej
 */
public class PrivChat extends Frame {
    private static final int CHPANELW = 115;
    private static final int CHPANELH = 35;
    private static final int NENTRYX = 35;
    private static final int NENTRYW = 135;
    private static final int NENTRYH = 20;
    private static final int SMENUPOS = 16;
    private static final Color COLOR1 = new Color(158, 204, 255);
    private static final Color COLOR2 = new Color(255, 255, 230);
    TextEntry nickEntry = new TextEntry(new TextCommand() {
                public void execute(String txt) {
                    chatpanel.append(txt, med.getNick());
                    med.writeToSock(txt, nick);
                }
            });
    ChatPanel chatpanel;
    SMenu smenu;
    SmileButton sbutton;
    Mediator med;
    String nick;

    /**
     * Creates a new PrivChat object.
     *
     * @param med Mediator object
     * @param nick remote nickname
     */
    public PrivChat(Mediator med, String nick) {
        super(med.getProperties().getProperty("rozmowa") + " " + nick);
        this.nick = nick;
        this.med = med;

        chatpanel = new ChatPanel(med, Mediator.WIDTH - CHPANELW,
                Mediator.HEIGHT - CHPANELH);
        smenu = new SMenu(med, nickEntry);
        sbutton = new SmileButton(med.getSmiles().getSmile1(), smenu);

        show();
        setLayout(null);
        setResizable(false);

        add(chatpanel);
        chatpanel.setText(med.getProperties().getProperty("rozmowa") + " " +
            nick);
        chatpanel.setLocation(getInsets().left, getInsets().top);

        nickEntry.setBounds(getInsets().left,
            Mediator.HEIGHT - NENTRYX + getInsets().top,
            Mediator.WIDTH - NENTRYW, NENTRYH);
        nickEntry.setBackground(COLOR1);
        add(nickEntry);

        sbutton.setBounds(nickEntry.getSize().width + getInsets().left,
            nickEntry.getLocation().y, NENTRYH, NENTRYH);
        add(sbutton);

        add(smenu);
        smenu.setVisible(false);
        smenu.setLocation((chatpanel.getSize().width +
            chatpanel.getLocation().x) - smenu.getSize().width - SMENUPOS,
            (chatpanel.getSize().height + chatpanel.getLocation().y) -
            smenu.getSize().height);

        setSize(chatpanel.getSize().width + getInsets().left +
            getInsets().right,
            chatpanel.getSize().height + NENTRYH + getInsets().bottom +
            getInsets().top);

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    setVisible(false);
                }
            });
    }

    /**
     * Shows the window.
     */
    public void show() {
        //super.setVisible(true);
        super.show();
        chatpanel.setVisible(true);
        nickEntry.setVisible(true);
        sbutton.setVisible(true);
    }

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    public String toString() {
        return nick;
    }

    /**
     * Appends a new server message.
     *
     * @param msg message to be appended
     */
    public void addServerMsg(String msg) {
        chatpanel.append(msg);
    }

    /**
     * Appends a new message said by remote nickname.
     *
     * @param msg message to be appended
     */
    public void addRemoteMsg(String msg) {
        chatpanel.append(msg, nick, COLOR2);
    }

    /**
     * This method is invoked whenever remote nickname changes.
     *
     * @param newnick new nickname
     */
    public void setNewNick(String newnick) {
        nick = newnick;
        setTitle(med.getProperties().getProperty("rozmowa") + " " + nick);
    }
}
