package madqba.net;

import madqba.awt.chat.Mediator;
import madqba.awt.chat.PrivChat;

import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import java.net.Socket;

import java.util.Enumeration;
import java.util.StringTokenizer;


/**
 * This is a Thread which handles all messages which come from server.
 *
 * @author Jakub Madej
 */
public class ClientProcess extends Thread {
    private static Logger log = Logger.getRootLogger();
    private OutputStreamWriter out;
    private BufferedReader in;
    private Mediator med;

    private final static String JOIN_CMD = "JOIN";
    private final static String QUIT_CMD = "QUIT";
    private final static String PART_CMD = "PART";
    private final static String KICK_CMD = "KICK";
    private final static String NICK_CMD = "NICK";
    private final static String PRIVMSG_CMD = "PRIVMSG";

    /**
     * Creates a new ClientProcess object.
     *
     * @param med Mediator object
     * @param s Socket object
     */
    public ClientProcess(Mediator med, Socket s) {
        this.med = med;

        try {
            in = new BufferedReader(new InputStreamReader(s.getInputStream()));
            out = new OutputStreamWriter(s.getOutputStream());
        } catch (Exception ex) {
            med.connError(s);

            return;
        }

        start();
    }

    /**
     * This method makes a loop in whcich the client checks for new messages
     * from server.
     */
    public void run() {
        for (;;) {
            String msg = null;
            String cmd = "";

            try {
                msg = in.readLine();
            } catch (Exception ex) {
                med.connLost();

                return;
            }

            if (msg != null) {
                log.info(msg);

                if (msg.startsWith("PING :")) {
                    try {
                        out.write("PONG " + msg.substring(6) + "\r\n");
                        out.flush();
                    } catch (Exception ex) {
                        med.connLost();

                        return;
                    }

                    continue;
                }

                cmd = msg.substring(msg.indexOf(" ") + 1,
                        msg.indexOf(" ", msg.indexOf(" ") + 1));

                if (cmd.equals(JOIN_CMD) &&
                        !msg.substring(1, msg.indexOf("!")).equals(med.getNick()))
                    przychodzi(msg.substring(1, msg.indexOf("!")));
                else if (cmd.equals(PART_CMD) || cmd.equals(QUIT_CMD))
                    odchodzi(msg.substring(1, msg.indexOf("!")));
                else if (cmd.equals(KICK_CMD)) {
                    String oldnick = "";
                    StringTokenizer strtok = new StringTokenizer(msg);

                    for (int i = 0; i < 4; i++)
                        oldnick = strtok.nextToken();

                    if (oldnick.equals(med.getNick())) {
                        med.connLost();

                        return;
                    } else {
                        med.removeFromList(oldnick);
                        med.addMsg(oldnick + " " +
                            med.getProperties().getProperty("odchodzi") + ".");
                    }
                } else if (cmd.equals(NICK_CMD))
                    nickChange(msg);
                else if (cmd.equals("311"))
                    whois(msg);
                else if (cmd.equals("319"))
                    isOp(msg);
                else if (cmd.equals("401"))
                    niedostepny(msg);
                else if (cmd.equals(PRIVMSG_CMD)) {
                    StringTokenizer strtok = new StringTokenizer(msg);
                    String oldnick = "";

                    for (int i = 0; i < 3; i++)
                        oldnick = strtok.nextToken();

                    if (oldnick.equals(med.getChan())) {
                        med.addMsg(msg.substring(msg.indexOf(":") + 1,
                                msg.indexOf("!")),
                            msg.substring(msg.indexOf(":", 2) + 1));

                        continue;
                    }

                    try {
                        if (msg.substring(msg.indexOf(""), msg.indexOf("") +
                                    9).equals("VERSION")) {
                            try {
                                out.write("NOTICE " +
                                    msg.substring(1, msg.indexOf("!")) +
                                    " :VERSION java irc client v1.0, (c) 2003 by Jakub Madej <madqba@atari.pl>\r\n");
                                out.flush();
                            } catch (Exception ex) {
                                med.connLost();

                                return;
                            }

                            continue;
                        }
                    } catch (Exception exc) {
                    }

                    String sender = msg.substring(1, msg.indexOf("!"));
                    PrivChat priv = null;
                    Enumeration en = med.getPrivList().elements();

                    while (en.hasMoreElements()) {
                        Object obj = en.nextElement();

                        if (obj.toString().equals(sender)) {
                            priv = (PrivChat) obj;

                            break;
                        }
                    }

                    if (priv == null) {
                        priv = new PrivChat(med, sender);
                        med.getPrivList().insertElementAt(priv, 0);
                    }

                    priv.show();
                    priv.addRemoteMsg(msg.substring(msg.indexOf(":", 2) + 1));
                }
            }
        }
    }

    /**
     * This method is invoked when new person joins the channel.
     *
     * @param kto nickname of joining person
     */
    private void przychodzi(String kto) {
        med.addToList(kto);
        med.addMsg(med.getProperties().getProperty("przychodzi") + " " + kto +
            ".");
    }

    /**
     * This method is invoked when someone leaves the channel.
     *
     * @param kto nickname of leaving person
     */
    private void odchodzi(String kto) {
        med.removeFromList(kto);
        med.addMsg(kto + " " + med.getProperties().getProperty("odchodzi") +
            ".");
    }

    /**
     * This method is invoked when somebody changes his nickname.
     *
     * @param msg the whole message sent by server
     */
    private void nickChange(String msg) {
        String oldnick = msg.substring(1, msg.indexOf("!"));
        String newnick = msg.substring(msg.indexOf(":", 2) + 1);
        med.removeFromList(oldnick);
        med.addToList(newnick);

        Enumeration en = med.getPrivList().elements();
        PrivChat pv = null;

        while (en.hasMoreElements()) {
            Object item = en.nextElement();

            if (item.toString().equals(oldnick))
                pv = (PrivChat) item;
        }

        if (pv != null)
            pv.setNewNick(newnick);

        med.addMsg(oldnick + " " + med.getProperties().getProperty("zmienia") +
            " " + newnick);
    }

    /**
     * This method is invoked when server responds to a "whois" message.
     *
     * @param msg the whole message sent by server
     */
    private void whois(String msg) {
        StringTokenizer strtok = new StringTokenizer(msg);
        String wnick = null;

        for (int i = 0; i < 4; i++)
            wnick = strtok.nextToken();

        med.addMsg("Info o " + wnick + ": " +
            msg.substring(msg.indexOf(":", 2) + 1));
    }

    /**
     * This method is invoked when server responds to a "whois" message and
     * corresponding person is a channel operator.
     *
     * @param msg the whole message sent by server
     */
    private void isOp(String msg) {
        StringTokenizer strt = new StringTokenizer(msg);
        String wnick = null;

        for (int i = 0; i < 4; i++)
            wnick = strt.nextToken();

        StringTokenizer strtok = new StringTokenizer(msg.substring(msg.indexOf(
                        ":", 2) + 1));
        boolean isop = false;

        while (strtok.hasMoreTokens()) {
            String chan = strtok.nextToken();

            if ((chan.length() == 7) && chan.startsWith("@") &&
                    chan.substring(1).equals(med.getChan()))
                isop = true;
        }

        if (isop)
            med.addMsg(wnick + " " + med.getProperties().getProperty("op"));
    }

    /**
     * This method is invoked whenever client sends a messages to a person who
     * left, and server respond witch a proper message.
     *
     * @param msg the message
     */
    private void niedostepny(String msg) {
        StringTokenizer strtok = new StringTokenizer(msg);
        String oldnick = "";

        for (int i = 0; i < 4; i++)
            oldnick = strtok.nextToken();

        Enumeration en = med.getPrivList().elements();

        while (en.hasMoreElements()) {
            Object item = en.nextElement();

            if (item.toString().equals(oldnick))
                ((PrivChat) item).addServerMsg(oldnick + " " +
                    med.getProperties().getProperty("niedost"));
        }
    }
}
